      Program Deg2Kmdp

* Given a lat and long, computes km N and E of a reference point

* Sample input (col. 1 starts after the initial "*")

*Name of output file:
*  deg2km.out
*Coordinates (title, ref. coord. (-west), n, e dist to point (km)):
*  JAB
* 34.31100 -118.49667  Ref for JAB
* 34.31151 -118.49665  coords of JAB
*  JGB 
* 34.31311 -118.49914  Ref for JGB
* 34.31299 -118.49884  coords of JGB
* stop


* Dates: 12/06/98 - Written by D.M. Boore, based on deg2km
*        12/18/98 - Appended version of deg2km_f modified to
*                   allow for negative latitudes
*        09/16/00 - assume -1. for sign of west longitude
*        04/30/02 - suggest default control filename

      character buf*70, f_in*30, f_out*30
      double precision vn, ve, alat_ref, along_ref, vlat, vlong

      logical f_in_exist

      f_in_exist = .false.
      do while (.not. f_in_exist)
        f_in = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = DEG2KMDP.CTL;'//
     :    ' CTL-BRK to quit): '
        read(*, '(a)') f_in
        if (f_in(1:4) .eq. '    ') f_in = 'deg2kmdp.ctl'
        inquire(file=f_in, exist=f_in_exist)
        if (.not. f_in_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      nu_in = 10
      open(nu_in, file=f_in, status = 'unknown')
      write(*, '(1x,a)') f_in

* Obtain info from control file:

      read(nu_in,*)                            ! skip comment line
      f_out = ' '
      read(nu_in, '(a)') f_out
      nu_out = 20
      open(nu_out, file=f_out, status = 'unknown')
      
      read(nu_in,*)                            ! skip comment line

1000  continue

      buf = ' '
      read(nu_in, '(a)') buf                   ! read title or stop
      write(*, '(1x,a)') buf
      write(nu_out,*)
      write(nu_out,'(a)') buf
      call upstr(buf)
      if(buf(2:5) .eq. 'STOP') then
        close(nu_in)
        close(nu_out)
        stop
      end if
      
      read(nu_in,*) alat_ref, along_ref        ! reference coordinates
      read(nu_in,*) vlat, vlong                ! coords of point

      write(*,'(a,1x,f8.4)') ' alat_ref = ', alat_ref
      call deg2km_f( vlat, vlong, alat_ref, along_ref, vn, ve )

      write(*,'(a,2(1x,f10.5))') ' vn, ve = ', vn, ve

      write(nu_out,'(a,2(1x,f10.5))') ' Ref. lat, long. = ',
     : alat_ref, along_ref

      write(nu_out,'(a,2(1x,f10.5))') ' vlat, vlong = ', 
     : vlat, vlong

      write(nu_out,'(a,2(1x,f10.5))') 
     : ' Using Richter tables: Distance north, east (km) = ',
     : vn, ve

      pi = 4.0*atan(1.0)
      d2r = pi/180.0

      vn = (vlat - alat_ref)*d2r*6371.0
      ve = (vlong - along_ref)*d2r*cos(0.5*(vlat+alat_ref)*d2r)*6371.0
      write(nu_out,'(a,2(1x,f10.5))') 
     : ' Using sphere approx : Distance north, east (km) = ',
     : vn, ve

      go to 1000

      end

*-------------------- BEGIN DEG2KM_F ----------------------------
      subroutine deg2km_f( alat_sta, along_sta, alat_ref, along_ref, 
     :                       d_sta_n, d_sta_e   )
        
* convert lat, long into km north and east from a reference point

* assumes latitude between 0 and 70 degrees
* assumes west longitude is negative
* assumes angles in degrees

* WARNING: NEEDS DOUBLE PRECISION VERSION OF LOCATE (ATTACHED HERE)

* Dates:  12/06/98 - written by D. Boore, based on km2deg_f
*         12/18/98 - modified to allow for negative latitudes
      
      double precision alat_tbl(71), b_tbl(71), adcoslat_tbl(71)
      double precision   a, b, dphi, dlambda,
     :                          d_sta_n_d,   d_sta_e_d, 
     :                         alat_ref_d, along_ref_d, 
     :                         alat_sta_d, along_sta_d
      Data alat_tbl /
     : 0.000000, 1.000000, 2.000000, 3.000000, 4.000000, 5.000000,
     : 6.000000, 7.000000, 8.000000, 9.000000,10.000000,11.000000,
     :12.000000,13.000000,14.000000,15.000000,16.000000,17.000000,
     :18.000000,19.000000,20.000000,21.000000,22.000000,23.000000,
     :24.000000,25.000000,26.000000,27.000000,28.000000,29.000000,
     :30.000000,31.000000,32.000000,33.000000,34.000000,35.000000,
     :36.000000,37.000000,38.000000,39.000000,40.000000,41.000000,
     :42.000000,43.000000,44.000000,45.000000,46.000000,47.000000,
     :48.000000,49.000000,50.000000,51.000000,52.000000,53.000000,
     :54.000000,55.000000,56.000000,57.000000,58.000000,59.000000,
     :60.000000,61.000000,62.000000,63.000000,64.000000,65.000000,
     :66.000000,67.000000,68.000000,69.000000,70.000000
     :/
      Data b_tbl /
     : 1.842808, 1.842813, 1.842830, 1.842858, 1.842898, 1.842950,
     : 1.843011, 1.843085, 1.843170, 1.843265, 1.843372, 1.843488,
     : 1.843617, 1.843755, 1.843903, 1.844062, 1.844230, 1.844408,
     : 1.844595, 1.844792, 1.844998, 1.845213, 1.845437, 1.845668,
     : 1.845907, 1.846153, 1.846408, 1.846670, 1.846938, 1.847213,
     : 1.847495, 1.847781, 1.848073, 1.848372, 1.848673, 1.848980,
     : 1.849290, 1.849605, 1.849992, 1.850242, 1.850565, 1.850890,
     : 1.851217, 1.851543, 1.851873, 1.852202, 1.852531, 1.852860,
     : 1.853188, 1.853515, 1.853842, 1.854165, 1.854487, 1.854805,
     : 1.855122, 1.855433, 1.855742, 1.856045, 1.856345, 1.856640,
     : 1.856928, 1.857212, 1.857490, 1.857762, 1.858025, 1.858283,
     : 1.858533, 1.858775, 1.859008, 1.859235, 1.859452
     :/
      Data adcoslat_tbl /
     : 1.855365, 1.855369, 1.855374, 1.855383, 1.855396, 1.855414,
     : 1.855434, 1.855458, 1.855487, 1.855520, 1.855555, 1.855595,
     : 1.855638, 1.855683, 1.855733, 1.855786, 1.855842, 1.855902,
     : 1.855966, 1.856031, 1.856100, 1.856173, 1.856248, 1.856325,
     : 1.856404, 1.856488, 1.856573, 1.856661, 1.856750, 1.856843,
     : 1.856937, 1.857033, 1.857132, 1.857231, 1.857331, 1.857435,
     : 1.857538, 1.857643, 1.857750, 1.857858, 1.857964, 1.858074,
     : 1.858184, 1.858294, 1.858403, 1.858512, 1.858623, 1.858734,
     : 1.858842, 1.858951, 1.859061, 1.859170, 1.859276, 1.859384,
     : 1.859488, 1.859592, 1.859695, 1.859798, 1.859896, 1.859995,
     : 1.860094, 1.860187, 1.860279, 1.860369, 1.860459, 1.860544,
     : 1.860627, 1.860709, 1.860787, 1.860861, 1.860934
     :/

      pi = 4.0*atan(1.0)
      dtor = pi/ 180.0

      write(*,'(a,1x,f8.4)') ' alat_ref = ', alat_ref
      alat_ref_d = alat_ref
      along_ref_d = along_ref

      alat_sta_d = alat_sta
      along_sta_d = along_sta

* interpolate to find proper arc distance:

      write(*,'(a,1x,f8.4)') ' alat_ref = ', alat_ref
      write(*,'(a,1x,f8.4)') ' alat_ref_d = ', alat_ref_d
      call locate_d( alat_tbl, 71, abs(alat_ref_d), j)
      write(*,'(a,1x,f10.5,1x,i3)') 
     : ' alat_ref_d, j = ',  alat_ref_d, j 
      b = b_tbl(j) + (abs(alat_ref_d)-alat_tbl(j))*
     :  (b_tbl(j+1)-b_tbl(j))/
     :  (alat_tbl(j+1)-alat_tbl(j))

      adcoslat = adcoslat_tbl(j) + (abs(alat_ref_d)-alat_tbl(j))*
     :  (adcoslat_tbl(j+1)-adcoslat_tbl(j))/
     :  (alat_tbl(j+1)-alat_tbl(j))

      a = adcoslat * cos(dtor*abs(alat_ref_d))

* compute lat,long relative to reference:
      dphi = alat_sta_d - alat_ref_d
      dlambda = along_sta_d - along_ref_d

* convert from degrees to minutes of arc:
      dlambda = dlambda * 60.0
      dphi    = dphi    * 60.0

* compute distances (positive ve corresponds to increase in longitude:
*                    vn positive to the north, ve positive to the east)
      d_sta_e_d =  a * dlambda 
*      d_sta_e_d = -a * dlambda ! used when assume west is positive
      d_sta_n_d =  b * dphi

      d_sta_n = d_sta_n_d
      d_sta_e = d_sta_e_d

      return
      end
*-------------------- END DEG2KM_F ----------------------------

*-------------------- BEGIN LOCATE_D ----------------------------
      SUBROUTINE locate_d(xx,n,x,j)   ! Double precision version of locate
      INTEGER j,n
      double precision x,xx(n)
      INTEGER jl,jm,ju
      jl=0
      ju=n+1
10    if(ju-jl.gt.1)then
        jm=(ju+jl)/2
        if((xx(n).ge.xx(1)).eqv.(x.ge.xx(jm)))then
          jl=jm
        else
          ju=jm
        endif
      goto 10
      endif
      if(x.eq.xx(1))then
        j=1
      else if(x.eq.xx(n))then
        j=n-1
      else
        j=jl
      endif
      return
      END
* --------------------- END LOCATE_D -----------------

* --------------------- BEGIN UPSTR ----------------------------------
      Subroutine UPSTR ( text )
* Converts character string in TEXT to uppercase
* Dates: 03/12/96 - Written by Larry Baker

C
      Implicit   None
C
      Character  text*(*)
C
      Integer    j
      Character  ch
C
      Do 1000 j = 1,LEN(text)
         ch = text(j:j)
         If ( LGE(ch,'a') .and. LLE(ch,'z') ) Then
            text(j:j) = CHAR ( ICHAR(ch) - ICHAR('a') + ICHAR('A') )
         End If
 1000    Continue
C
      Return
      End
* --------------------- END UPSTR ----------------------------------
